module DataWriter(
					x_addr, //x coordinate of pixel to draw to
					y_addr, //y coordinate of pixel to draw to
					color_data, //the color data
					isFinished, //flag if the write has completed
					clock, //the clock to handle the writing
					sync, //to keep us in sync with VGA
					reset, // a reset signal
					write_req_clk, //clock that starts a new write request
					read_addr_out, //the address we want to read data from
					data_write_out, //the new data that we will write to SRAM
					we_out, //the write enable signal (0 = write, 1 = read)
					read_data_in, //the data that was read
					currFrame_out,
					frame_sel,
					drawMode_sel,
					Coord_X,
					Coord_Y

);

input wire clock, reset, sync;
input wire [9:0] Coord_X, Coord_Y;
input wire write_req_clk;

input wire [2:0] drawMode_sel;
input wire frame_sel;

input wire [9:0] x_addr;
input wire [8:0] y_addr;
input wire [15:0] color_data;
input wire [15:0] read_data_in;

output wire [1:0] currFrame_out;
assign currFrame_out = {frame_sel,frame_sel};

output reg isFinished;
output reg [17:0] read_addr_out;
output reg we_out;
output reg [15:0] data_write_out;


parameter [2:0]	sixteenBit_single_320x240 = 3'd0,
				eightBit_single_640x480 = 3'd1,
				eightBit_double_320x240 = 3'd2,
				fourBit_double_640x480 = 3'd3,
				fiveBit_triple_320x240 = 3'd4,
				fourBit_quad_320x240 = 3'd5,
				twoBit_quad_640x480 = 3'd6,
				foreground_background = 3'd7;
				
reg [1:0] writeMode;


parameter [1:0]	waitForRequest = 2'd0,
				loadData = 2'd1,
				writeData = 2'd2,
				sendACK = 2'd3;
				

reg [15:0] color_formated;
reg wasWriting;



always @(posedge clock) begin
	if(reset) begin
		isFinished <= 1'b1;
		writeMode <= waitForRequest;
		read_addr_out <= 0;
		we_out <= 1'b1;
		data_write_out <= 0;
		color_formated <= 0;
		case(drawMode_sel)
			sixteenBit_single_320x240: begin read_addr_out <= {Coord_X[9:1],Coord_Y[9:1]} ;end
			eightBit_single_640x480: begin read_addr_out <= {Coord_X[9:1],Coord_Y[8:0]/*Coord_Y[9:1]*/} ;end
			eightBit_double_320x240: begin read_addr_out <= {Coord_X[9:1],Coord_Y[9:1]} ;end
			fourBit_double_640x480: begin read_addr_out <= {Coord_X[9:1],Coord_Y[8:0]/*Coord_Y[9:1]*/} ;end
		endcase
		
			we_out <= 1'b0;
			data_write_out <= 16'hFFFF;
			wasWriting <= 1'b0;
	end
	else begin
		if(sync) begin
			case(writeMode)
				waitForRequest: begin
					isFinished <= 1'b0;
					if(write_req_clk | wasWriting) begin
						writeMode <= loadData;
						wasWriting <= 1'b1;
						case(drawMode_sel)
							sixteenBit_single_320x240: 	begin color_formated <= color_data; end
							eightBit_single_640x480: 	begin color_formated <= {color_data[15:13],color_data[11:9],color_data[7:6],color_data[15:13],color_data[11:9],color_data[7:6]}; end//RRRGGGBB
							eightBit_double_320x240:	begin color_formated <= {color_data[15:13],color_data[11:9],color_data[7:6],color_data[15:13],color_data[11:9],color_data[7:6]}; end//RRRGGGBB
							fourBit_double_640x480: 	begin color_formated <= {4{color_data[15],color_data[11:10],color_data[7]}}; end //RGGB
							fiveBit_triple_320x240: 	begin color_formated <= {({3{color_data[15:14],color_data[11:10],color_data[7]}}),1'b0}; end //RRGGB
							fourBit_quad_320x240: 		begin color_formated <= {4{color_data[15],color_data[11:10],color_data[7]}}; end //RGGB
							twoBit_quad_640x480: 		begin color_formated <= 0; end
							foreground_background: 		begin color_formated <= 0; end
							default: 					begin color_formated <= 16'h0F00; end
						endcase
					end
				end
				loadData: begin //TODO BOUND CHECK
					we_out <= 1'b1; //we want to read
					case(drawMode_sel)
						sixteenBit_single_320x240: 	begin read_addr_out <= {x_addr[8:0],y_addr[8:0]}; end
						eightBit_single_640x480: 	begin read_addr_out <= {x_addr[9:1],y_addr[8:0]}; end //{1'b0,y_addr[8:1]}}; end
						eightBit_double_320x240: 	begin read_addr_out <= {x_addr[8:0],y_addr[8:0]}; end
						fourBit_double_640x480: 	begin read_addr_out <= {x_addr[9:1],y_addr[8:0]}; end
						fiveBit_triple_320x240: 	begin read_addr_out <= {x_addr[8:0],y_addr[8:0]}; end
						fourBit_quad_320x240: 		begin read_addr_out <= {x_addr[8:0],y_addr[8:0]}; end
						twoBit_quad_640x480: 		begin read_addr_out <= {x_addr[9:1],{1'b0,y_addr[8:1]}}; end
						foreground_background: 		begin read_addr_out <= {x_addr[8:0],y_addr[8:0]}; end//CHANGE
						default:					begin read_addr_out <= 0; end
					endcase
					writeMode <= writeData;
					wasWriting <= 1'b1;
				end
				writeData: begin
					we_out <= 1'b0; //we want to write
					case(drawMode_sel) //assuming color_data is already formatted
						sixteenBit_single_320x240: 	begin data_write_out <= color_formated; end
						eightBit_single_640x480: 	begin data_write_out <= (x_addr[0] == 1'b1) ? {(read_data_in & 16'hFF00) | (16'h00FF & color_formated)} : {(read_data_in & 16'h00FF) | (16'hFF00 & color_formated)}; end
						eightBit_double_320x240: 	begin data_write_out <= (frame_sel == 1'b1) ? {(read_data_in & 16'hFF00) | (16'h00FF & color_formated)} : {(read_data_in & 16'h00FF) | (16'hFF00 & color_formated)}; end
						fourBit_double_640x480: 	begin data_write_out <= x_addr[0] ? 
																					(frame_sel ? ((read_data_in & 16'hFF0F) | (16'h00F0 & color_formated)) : ((read_data_in & 16'hFFF0) | (16'h000F & color_formated)) ) : 
																					(frame_sel ? ((read_data_in & 16'hF0FF) | (16'h0F00 & color_formated)) : ((read_data_in & 16'h0FFF) | (16'hF000 & color_formated)) ); end
						fiveBit_triple_320x240: 	begin data_write_out <= color_data; end //TODO
						fourBit_quad_320x240: 		begin data_write_out <= color_data; end //TODO
						twoBit_quad_640x480: 		begin data_write_out <= color_data; end //TODO
						foreground_background: 		begin data_write_out <= color_data; end //TODO
						default: 					begin data_write_out <= 16'h00F0; end
					endcase
					writeMode <= sendACK;
					wasWriting <= 1'b0;
				end
				sendACK: begin
					we_out <= 1'b1; //not writing anymore, so set to read to be safe
					isFinished <= 1'b1;
					writeMode <= waitForRequest;
					wasWriting <= 1'b0;
				end
			endcase
		end
		else
		begin
			case(drawMode_sel)
				sixteenBit_single_320x240: begin read_addr_out <= {Coord_X[9:1],Coord_Y[9:1]} ;end
				eightBit_single_640x480: begin read_addr_out <= {Coord_X[9:1],Coord_Y[8:0]/*Coord_Y[9:1]*/} ;end
				eightBit_double_320x240: begin read_addr_out <= {Coord_X[9:1],Coord_Y[9:1]} ;end
				fourBit_double_640x480: begin read_addr_out <= {Coord_X[9:1],Coord_Y[8:0]/*Coord_Y[9:1]*/} ;end
			endcase
			writeMode <= waitForRequest;//loadData;
			//isFinished <= 1'b1;
			we_out <= 1'b1;
		end
	end
end




endmodule

