module PacketManager_updated(
							data_clock,
							data_in,
							latestPacket,
							packetCRC,
							packetIsAvailable,
							reset,
							grab_clock,
							fast_clock,
							PR,
							packetID
							);
							
input wire data_clock;
input wire [7:0] data_in;
input wire reset;
input wire grab_clock;
input wire fast_clock;

output wire [79:0] latestPacket;
output wire packetCRC;
output wire packetIsAvailable;
output wire PR;
assign PR = packetReady;


reg packetReady, packetIsAvail;
reg [79:0] currentPacket, lastPacket;
reg [3:0] packetIndex;
output reg [7:0] packetID;
reg CRC;

reg [2:0] state;

parameter [2:0] 	waiting = 3'd0,
					looping = 3'd1,
					waitForGrab = 3'd2;


//assign packetIsAvailable = packetIsAvail ^ packetReady;
assign packetIsAvailable = packetReady;
assign latestPacket = lastPacket;
assign packetCRC = CRC;


always @(posedge fast_clock) begin
	if(reset | ~data_clock) begin
		packetIsAvail <= 1'b0;
	end
	else begin
		if(grab_clock) begin
			packetIsAvail <= 1'b1;
		end
		
	end
end

always @(posedge data_clock, posedge reset)
begin
	if(reset) begin
		currentPacket <= 80'b0;
		packetIndex <= 4'b0;
		packetReady <= 1'b0;
		lastPacket <= 80'b0;
		CRC <= 1'b0;
		state <= waiting;
		packetID <= 0;
	end
	else begin
		case(state)
			waiting: begin
				currentPacket <= currentPacket | {72'b0, data_in};
				packetIndex <= packetIndex + 1;
				state <= looping;
				packetReady <= 1'b0;
			end
			looping: begin
				if(packetIndex < 4'd9) begin
					currentPacket <= ( (currentPacket << 8) | {72'b0,data_in} );
					packetIndex <= packetIndex + 1;
				end
				else begin //the last packet to arrive
					lastPacket <= ( (currentPacket << 8) | {72'b0,data_in} );
					CRC <= (data_in == (currentPacket[7:0] ^ currentPacket[15:8] ^ currentPacket[23:16]
							^ currentPacket[31:24] ^ currentPacket[39:32] ^ currentPacket[47:40]
							^ currentPacket[55:48] ^ currentPacket[63:56] ^ currentPacket[71:64])) ?
								1'b1: 1'b0;
					packetReady <= 1'b1;
					state <= waiting;
					currentPacket <= 80'b0;
					packetIndex <= 4'd0;
					if(packetID < 8'd255) begin
						packetID <= packetID + 8'd1;
					end
					else begin
						packetID <= 8'd0;
					end
					
				end
			end
			waitForGrab: begin
				//if(grab_clock) begin
					state <= waiting;
					packetReady <= 1'b0;
				//end
			end
		endcase
	
	end
end


endmodule

