//SRAM Multiplexer

module SRAM_mux(
				CF_addr, CF_data, CF_we,
				CD_addr, CD_data, CD_we,
				SF_addr, SF_data, SF_we,
				SD_addr, SD_data, SD_we,
				LD_addr, LD_data, LD_we,
				TW_addr, TW_data, TW_we,
				FU_addr, FU_data, FU_we,
				IDLE_addr, IDLE_data, IDLE_we,
				VGA_CLK,
				draw_select,
				addr_reg, data_reg, we,
				sync, Coord_X, Coord_Y
);

input wire [17:0] CF_addr, CD_addr, SD_addr, SF_addr, LD_addr, IDLE_addr, TW_addr, FU_addr;
input wire [15:0] CF_data, CD_data, SD_data, SF_data, LD_data, IDLE_data, TW_data, FU_data;
input wire CF_we, CD_we, SD_we, SF_we, LD_we, IDLE_we, TW_we, FU_we;
input wire VGA_CLK;
input wire sync;
input wire [9:0] Coord_X, Coord_Y;


input wire [3:0] draw_select;

output reg [17:0] addr_reg;
output reg [15:0] data_reg;
output reg we;

parameter 	[3:0] 	circle_fill = 4'd0,
					circle_draw = 4'd1,
					square_fill = 4'd2,
					square_draw = 4'd3,
					line_draw = 4'd4,
					text_write = 4'd5,
					frame_update = 4'd6,
					idle = 4'd15;
			
/*			
assign addr_reg = 	
					(draw_select == circle_fill) ? CF_addr :
					(draw_select == circle_draw) ? CD_addr :
					(draw_select == square_fill) ? SF_addr :
					(draw_select == square_draw) ? SD_addr :
					(draw_select == line_draw) ? LD_addr :
					18'd0;
					
assign data_reg = 	
					(draw_select == circle_fill) ? CF_data :
					(draw_select == circle_draw) ? CD_data :
					(draw_select == square_fill) ? SF_data :
					(draw_select == square_draw) ? SD_data :
					(draw_select == line_draw) ? LD_data :
					16'd0;	
					
assign we = 		
					(draw_select == circle_fill) ? CF_we :
					(draw_select == circle_draw) ? CD_we :
					(draw_select == square_fill) ? SF_we :
					(draw_select == square_draw) ? SD_we :
					(draw_select == line_draw) ? LD_we :
					1'b1;	*/			
			
			
always @ (posedge VGA_CLK) begin
	//if(sync) begin
		case(draw_select)
			circle_fill: begin
				we <= CF_we;
				addr_reg <= CF_addr;
				data_reg <= CF_data;
			end
			circle_draw: begin
				we <= CD_we;
				addr_reg <= CD_addr;
				data_reg <= CD_data;
			end
			square_fill: begin
				we <= SF_we;
				addr_reg <= SF_addr;
				data_reg <= SF_data;
			end
			square_draw: begin
				we <= SD_we;
				addr_reg <= SD_addr;
				data_reg <= SD_data;
			end
			line_draw: begin
				we <= LD_we;
				addr_reg <= LD_addr;
				data_reg <= LD_data;
			end
			text_write: begin
				we <= TW_we;
				addr_reg <= TW_addr;
				data_reg <= TW_data;
			end
			frame_update: begin
				we <= FU_we;
				addr_reg <= FU_addr;
				data_reg <= FU_data;
			end
			idle: begin
				addr_reg <= IDLE_addr;
				data_reg <= IDLE_data;
				we <= IDLE_we;
			end
			default: begin
				addr_reg <= 18'd0;
				data_reg <= 16'd0;
				we <= 1'b1;
			end
		endcase
	//end
	//else
	//begin
	//	addr_reg <= {Coord_X[9:1],Coord_Y[9:1]} ;
	//	we <= 1'b1;
	//end
end



endmodule

