module SerialTransmitter_updated( 
	input CLOCK_50,
	output  UART_TXD,
	input [7:0] toTransmit,
	input reset,
	input sendNew,
	input baud_clock,
	output sendInProgress
	);
	
	reg [7:0] txd, last_txd;
	reg [3:0] currBit;
	reg [3:0] transmitState;
	
	parameter [3:0] idleWait = 4'd0,
					sendStartBit = 4'd1,
					sendData = 4'd2,
					sendStopBit = 4'd3;
					
	reg txd_out;
	reg should_send;
	reg sendInProgressReg;
	
		
	assign UART_TXD = txd_out;
	assign sendInProgress = sendInProgressReg;
	
			
	always @(posedge CLOCK_50)
	begin
		if(sendNew & ~reset & ((transmitState == idleWait) )) begin
			should_send <= 1'b1;
		end
		if(reset) begin
			transmitState <= 4'd0;
			currBit <= 0;
			txd <= 0;
			txd_out <= 1;
			should_send <= 0;
			sendInProgressReg <= 0;
			
		end
		else begin
			if (baud_clock) //9600 times a second (aprox)
			begin
				
				
				case(transmitState)
					idleWait: begin
						txd_out <= 1'b1;
						sendInProgressReg <= 1'b0;
						if(should_send) begin
							txd <= toTransmit;
							last_txd <= toTransmit;
							transmitState <= sendStartBit;
							should_send <= 0;
						end
					end
					
					sendStartBit: begin
						txd_out <= 1'b0; //the start bit
						transmitState <= sendData;
						currBit <= 4'd0;
						sendInProgressReg <= 1'b1;
					end
					
					sendData: begin
						txd_out <= txd[currBit];
						currBit <= currBit + 4'd1;
						if(currBit == 4'd7) begin
							transmitState <= sendStopBit;
						end
					end
					
					sendStopBit: begin
						txd_out <= 1'b1;
						sendInProgressReg <= 1'b0;
						transmitState <= idleWait;
					end
										
				endcase
				
				
			end
			
		end
		
	end
endmodule
