//Peter Greczner, PC

module Processor(
                    clock_50,
                    draw_complete,
                    reset,
                    enable,
                    instruction,
                    pc,
                    isDrawInstruction
                    );


input wire clock_50;
input wire draw_complete;
input wire reset;
input wire enable;

output wire isDrawInstruction;
assign isDrawInstruction = isDraw;

input wire [79:0] instruction;
output wire pc;
assign pc = pc_out;


wire pc_enable;
assign pc_enable = enable & draw_complete;
wire [11:0] pc_in;
wire [11:0] pc_out;
ProgramCounter pc1(
                    .clock(clock_50),
                    .reset(reset),
                    .enabled(pc_enable),
                    .PC_out(pc_out),
                    .PC_in(pc_in),
                    .isDraw(isDraw),
                    .isDrawDone(draw_complete)
                    );
                    
                    
wire [3:0] ctrl_ALU;
wire [3:0] ctrl_BS;
wire isBranch, isStore, isJump, isDraw, isIMMOpp, isLoad;
wire [4:0] SA, SB, SD;
wire [31:0] IMM;
wire WE;
Decoder dec1(   .instruction(instruction), 
                .ctrl_ALU(ctrl_ALU), 
                .ctrl_BS(ctrl_BS), 
                .isBranch(isBranch),
                .isLoad(isLoad), 
                .isStore(isStore), 
                .isJump(isJump), 
                .isDraw(isDraw), 
                .isIMMOpp(isIMMOpp), 
                .SA(SA), 
                .SB(SB), 
                .SD(SD), 
                .IMM(IMM), 
                .WE(WE)
                );

wire [31:0] A_data, B_data, D_data;
RegisterFile regfile1(
                        .A_addr(SA), 
                        .B_addr(SB), 
                        .D_addr(SD), 
                        .w_en(WE), 
                        .A_data(A_data), 
                        .B_data(B_data), 
                        .D_data(D_data), 
                        .clk(clock_50)
                    );

wire [31:0] alu_res;
wire isZero, isNegative, isEqual, isGT, isLT;     
wire [31:0] B_input;
assign B_input = isIMMOpp ? IMM : B_data;               
ALU alu1(
            .control(ctrl_ALU), 
            .A(A_data), 
            .B(B_input), 
            .C(alu_res), 
            .isZero(isZero), 
            .isNegative(isNegative), 
            .isEqual(isEqual), 
            .isGT(isGT), 
            .isLT(isLT)
        );

wire [31:0] data_mem;     
assign D_data = isLoad ? data_mem : alu_res;   
DataRam dr1(
				.clock(clock_50), 
				.reset(reset), 
				.address(alu_res[7:0]), 
				.isStore(isStore), 
				.data_out(data_mem), 
				.data_in(B_data)
			);

wire takeBranch;        
BranchSelect bs1(
                    .isBranch(isBranch), 
                    .isJump(isJump), 
                    .ctrl_BS(ctrl_BS), 
                    .alu_EQ(isEqual), 
                    .alu_Z(isZero), 
                    .alu_GT(isGT), 
                    .alu_LT(isLT), 
                    .takeBranch(takeBranch)
                );
                
PCUpdater pcu1(
                .ctrl_BS(ctrl_BS), 
                .takeBranch(takeBranch), 
                .alu_res(alu_res), 
                .PC_now(pc_out), 
                .PC_next(pc_in)
                );

endmodule
