module TextWriter(	VGA_SYNC, VGA_CLK,
					data_reg,
					reset,
					my_id, id_req,
					command,
					color,
					font_select,
					letter_addr, letterX, letterY,
					available,
					
					xAddr, yAddr,
					write_clk,
					is_write_finished
					);
					
					
input wire VGA_SYNC, VGA_CLK;
//input wire [9:0] Coord_X, Coord_Y;
//output reg we; // we for SRAM
//output reg [17:0] addr_reg; //memory address register for SRAM
output reg [15:0] data_reg; //memory data register  for SRAM
input wire [15:0] color;
input wire [1:0] command;
input wire reset;
input wire [3:0] my_id, id_req;
input wire [1:0] font_select;
input wire [11:0] letter_addr;
input wire signed [10:0] letterX, letterY;
output wire available;
reg available_reg;
assign available = available_reg;

output reg write_clk;
input wire is_write_finished;

output reg [9:0] xAddr;
output reg [8:0] yAddr;


reg [11:0] currLetterAddr;
reg [2:0] widthInd;
wire [7:0] currLetterData;

font1_rom f1r(currLetterAddr, letterClock, currLetterData);

reg [2:0] lineNumber;
reg [2:0] state, next_state;
reg letterClock;

parameter 	[2:0] 	waiting = 3'd0,
					loadLetterLine = 3'd1,
					printLetterLine = 3'd2,
					looping = 3'd3,
					write_state = 3'd4,
					initial_clock = 3'd5;

always @ (posedge VGA_CLK) begin
	if(reset) begin
		//clear the screen
		//addr_reg <= {Coord_X[9:1],Coord_Y[9:1]} ;	// [17:0]
		//we <= 1'b0;								//write some memory
		data_reg <= 16'b0;						//write all zeros (black)
		available_reg <= 1'b0;
		lineNumber <= 3'd0;
		state <= waiting;
		widthInd = 3'd0;
		letterClock <= 1'b0;
	end
	else if (VGA_SYNC) begin
		if(1'b1) begin// (my_id == id_req)) begin
			case(state)
			
				write_state: begin
					if(is_write_finished) begin
						state <= next_state;
						write_clk <= 1'b0;
					end
				end
			
				waiting: begin
					if(command[1] == 1'b0) begin available_reg <= 1'b1; end
					//we <= 1'b1;
					if(command[1] == 1'b1 && (my_id == id_req)) begin
						available_reg <= 1'b0;
						lineNumber <= 3'd0;
						//state <= loadLetterLine;
						state <= initial_clock;
						letterClock <= 1'b1;
						currLetterAddr <= letter_addr;
						widthInd <= 3'd0;
						
					end
				end
				
				initial_clock: begin
					letterClock <= 1'b0;
					state <= loadLetterLine;
				end
				
				loadLetterLine: begin
					letterClock <= 1'b1;
					state <= looping;
				end
				
				looping: begin
					letterClock <= 1'b0;
					available_reg <= 1'b0;
					
					//Set the Printable Address
					//addr_reg <= {letterX[8:0] + widthInd,letterY[8:0] + lineNumber};
					
					
					//Draw the Letter
					if(currLetterData[~widthInd] == 1'b1) begin
						data_reg <= color;
						//we <= 1'b0;
						//xAddr <= {1'b0,letterX[8:0] + widthInd};
						xAddr <= letterX[9:0] + widthInd;
						yAddr <= letterY[8:0] + lineNumber;
						state <= write_state;
						write_clk <= 1'b1;
					end
					else begin //easy to implement w/background
						//we <= 1'b1;
						//data_reg <= 16'h0F00;
					end
					
					//Loop Through Pixels
					if(widthInd < 3'd7) begin
						widthInd <= widthInd + 3'd1;
						next_state <= looping;
						if(~currLetterData[~widthInd] == 1'b1) begin state <= looping; end
					end
					else begin
						if(lineNumber < 3'd7) begin
							lineNumber <= lineNumber + 3'd1;
							currLetterAddr <= letter_addr + {9'd0,(lineNumber+3'd1)};
							widthInd <= 3'd0;
							//state <= loadLetterLine;
							next_state <= loadLetterLine;
							if(~currLetterData[~widthInd] == 1'b1) begin state <= loadLetterLine; end
						end
						else begin
							//state <= waiting;
							next_state <= waiting;
							lineNumber <= 3'd0;
							if(~currLetterData[~widthInd] == 1'b1) begin state <= waiting; end
						end
					end
					
					
				end
				
				default: begin
					available_reg <= 1'b0;
					lineNumber <= 3'd0;
					state <= waiting;
					//we <= 1'b1;
				end
			
			endcase
		end
	end
	else
	begin
		//addr_reg <= {Coord_X[9:1],Coord_Y[9:1]} ;
		//we <= 1'b1;
	end
end




endmodule


